﻿#region Using

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Collections.Generic;
using BlogEngine.Core;
using System.Collections;

#endregion

namespace App_Code.Controls
{
	/// <summary>
	/// Builds a category list.
	/// </summary>
	public class LayeredCategoryList : Control
	{

        static LayeredCategoryList()
		{
			Post.Saved += delegate { _Html = null; };
			Category.Saved += delegate { _Html = null; };
		}

		#region Properties

		private static bool _ShowRssIcon = true;
		/// <summary>
		/// Gets or sets whether or not to show feed icons next to the category links.
		/// </summary>
		public bool ShowRssIcon
		{
			get { return _ShowRssIcon; }
			set
			{
				if (_ShowRssIcon != value)
				{
					_ShowRssIcon = value;
					_Html = null;
				}
			}
		}

		private bool _ShowPostCount;

		public bool ShowPostCount
		{
			get { return _ShowPostCount; }
			set
			{
				if (_ShowPostCount!= value)
				{
					_ShowPostCount = value;
					_Html = null;
				}
			}
		}


		private static object _SyncRoot = new object();
		private static string _Html;
		private string Html
		{
			get
			{
				if (_Html == null)
				{
					lock (_SyncRoot)
					{
						if (_Html == null)
						{
							HtmlGenericControl ul = BindCategories();
							System.IO.StringWriter sw = new System.IO.StringWriter();
							ul.RenderControl(new HtmlTextWriter(sw));
							_Html = sw.ToString();
						}
					}
				}

				return _Html;
			}
		}

		#endregion

		private HtmlGenericControl BindCategories()
		{
			SortedDictionary<string, Guid> dic = SortCategories();

			if (dic.Keys.Count == 0)
			{
				HtmlGenericControl none = new HtmlGenericControl("p");
				none.InnerText = "None";
				return none;
			}

			HtmlGenericControl ul = new HtmlGenericControl("ul");
			ul.ID = "layeredcategorylist";
            //<UL> Layered Stack.
            ArrayList ar = new ArrayList();
            ar.Add(ul);
            //Last li holder
            HtmlGenericControl lastElement = null;

			foreach (Guid id in dic.Values)
			{

                // Find full category
			    Category cat = Category.GetCategory(id);
			    string key = cat.CompleteTitle();

				HtmlGenericControl li = new HtmlGenericControl("li");
                if (lastElement == null) lastElement = li;
				if (ShowRssIcon)
				{
					HtmlImage img = new HtmlImage();
					img.Src = Utils.RelativeWebRoot + "pics/rssButton.gif";
					img.Alt = BlogSettings.Instance.SyndicationFormat.ToUpperInvariant() + " feed for " + key;
					img.Attributes["class"] = "rssButton";

					HtmlAnchor feedAnchor = new HtmlAnchor();
                    feedAnchor.HRef = cat.FeedRelativeLink;
					feedAnchor.Attributes["rel"] = "nofollow";
					feedAnchor.Controls.Add(img);

					li.Controls.Add(feedAnchor);
				}

				int posts = Post.GetPostsByCategory(dic[key]).FindAll(delegate(Post p)
				{
					return p.IsVisible;
				}).Count;

				string postCount = " (" + posts + ")";
				if (!ShowPostCount)
					postCount = null;

				HtmlAnchor anc = new HtmlAnchor();
                anc.HRef = cat.RelativeLink;
                /*
                 * Calc Parent Categories
                 */
                string akey = key;
                int parents = 1;
                if (key.Contains(" - ") == true)
                {
                    while (akey.IndexOf(" - ") != -1)
                    {
                        int idx = akey.IndexOf(" - ") + 3;
                        akey = akey.Substring(idx);

                        parents++;
                    }
                    //Layer Count Changed.
                    if ((parents) != ar.Count)
                    {
                        if (parents > ar.Count)
                        {
                            HtmlGenericControl parenthgc = lastElement;
                            while (parents > ar.Count)
                            {
                                HtmlGenericControl subul = new HtmlGenericControl("ul");
                                parenthgc.Controls.Add(subul);
                                parenthgc = subul;
                                ar.Add(subul);
                            }
                        }
                        else if (parents < ar.Count)
                        {
                            while (parents < ar.Count)
                            {
                                //RemoveStack
                                ar.RemoveAt(ar.Count - 1);
                            }
                        }
                    }
                    ((HtmlGenericControl)ar[ar.Count - 1]).Controls.Add(li);

                    li.Attributes.Add("class", "L" + (parents - 1));
                }
                else
                {
                    while (ar.Count != 1)
                    {
                        //RemoveStack
                        ar.RemoveAt(ar.Count - 1);

                    }
                }
                anc.InnerHtml = HttpUtility.HtmlEncode(akey) + postCount;
				anc.Title = "Category: " + key;

				li.Controls.Add(anc);
                if (key.Contains(" - ") == false)
                    ul.Controls.Add(li);
                lastElement = li;
            }

			return ul;
		}

		private SortedDictionary<string, Guid> SortCategories()
		{
			SortedDictionary<string, Guid> dic = new SortedDictionary<string, Guid>();
			foreach (Category cat in Category.Categories)
			{
			//	if (HasPosts(cat))
					dic.Add(cat.CompleteTitle(), cat.Id);
			}

			return dic;
		}

		private bool HasPosts(Category cat)
		{
			foreach (Post post in Post.Posts)
			{
				if (post.IsVisible)
				{
					foreach (Category category in post.Categories)
					{
						if (category == cat)
							return true;
					}
				}

			}
			return false;
		}

		/// <summary>
		/// Renders the control.
		/// </summary>
		public override void RenderControl(HtmlTextWriter writer)
		{
			writer.Write(Html);
			writer.Write(Environment.NewLine);
		}
	}
}