﻿#region Using

using System;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.IO;
using BlogEngine.Core;
using System.Collections.Generic;

#endregion

namespace App_Code.Controls
{
  /// <summary>
  /// Builds a page list.
  /// </summary>
  public class LayeredPageList : Control
  {

    /// <summary>
    /// Initializes the <see cref="PageList"/> class.
    /// </summary>
    static LayeredPageList()
    {
      BlogEngine.Core.Page.Saved += delegate { _Html = null; };
    }

    #region Properties
      

    private static object _SyncRoot = new object(); 
    private static string _Html;
    /// <summary>
    /// Caches the rendered HTML in the private field and first
    /// updates it when a post has been saved (new or updated).
    /// </summary>
    private string Html
    {
      get
      {
        if (_Html == null)
        {
          lock (_SyncRoot)
          {
              if (_Html == null || BlogEngine.Core.Page.Pages == null)
            {
              HtmlGenericControl ul = BindPages();
              System.IO.StringWriter sw = new System.IO.StringWriter();
              ul.RenderControl(new HtmlTextWriter(sw));
              _Html = sw.ToString();
            }
          }
        }

        return _Html;
      }
    }

    #endregion

    /// <summary>
    /// Loops through all pages and builds the HTML
    /// presentation.
    /// </summary>
    private HtmlGenericControl BindPages()
    {
        HtmlGenericControl ul = new HtmlGenericControl("ul");
        ul.Attributes.Add("class", "layeredpagelist");
        ul.ID = "layeredpagelist";
        int parents = 1;
        //親ページリストを取得
        List<BlogEngine.Core.Page> ps = GetStructurePageList(null);
        foreach (BlogEngine.Core.Page page in ps)
        {
            HtmlGenericControl li = new HtmlGenericControl("li");
            CreateStructureItem(li, page);
            li.Attributes.Add("CLASS", "L" + parents);
            //階層チェック
            if (page.HasChildPages == true)
            {
                CreateLayeredItem(li, page, (parents + 1));
           }
           ul.Controls.Add(li);
        }
        return ul;
        /*
         
        HtmlGenericControl ul = new HtmlGenericControl("dl");
        ul.Attributes.Add("class", "pagelist");
        ul.ID = "pagelist";

          foreach (BlogEngine.Core.Page page in BlogEngine.Core.Page.Pages)
          {
            if (page.ShowInList && page.IsVisibleToPublic)
            {
              HtmlGenericControl li = new HtmlGenericControl("li");

              HtmlAnchor anc = new HtmlAnchor();
              anc.HRef = page.RelativeLink.ToString();
              anc.InnerHtml = page.Title;
              anc.Title = page.Description;

              li.Controls.Add(anc);
              ul.Controls.Add(li);
            }
          }

      return ul;
         */
    }
    protected void CreateStructureItem(HtmlGenericControl ctl,BlogEngine.Core.Page page)
    {
        HtmlAnchor anc = new HtmlAnchor();
        anc.HRef = page.RelativeLink.ToString();
        anc.InnerHtml = page.Title;
        anc.Title = page.Description;
        ctl.Controls.Add(anc);

    }
    protected void CreateLayeredItem(HtmlGenericControl ctl, BlogEngine.Core.Page page,int lNumber)
    {
        HtmlGenericControl subul = new HtmlGenericControl("ul");
        List<BlogEngine.Core.Page> psc = GetStructurePageList(page);
        foreach (BlogEngine.Core.Page cpage in psc)
        {
            HtmlGenericControl subli = new HtmlGenericControl("li");
            CreateStructureItem(subli, cpage);
            subli.Attributes.Add("class", "L" + lNumber);
            if (cpage.HasChildPages == true)
            {
                CreateLayeredItem(subli, cpage,(lNumber+1));
            }
           subul.Controls.Add(subli);
        }
        ctl.Controls.Add(subul);
    }

    protected List<BlogEngine.Core.Page> GetStructurePageList(BlogEngine.Core.Page Parent)
    {
        List<BlogEngine.Core.Page> retval = new List<BlogEngine.Core.Page>();
        Guid gparent = new Guid();
        if (Parent == null)
        {
            gparent = Guid.Empty;
        }else{
            gparent = Parent.Id;
        }
            //トップ階層のリストを返す
          foreach (BlogEngine.Core.Page page in BlogEngine.Core.Page.Pages)
          {
            if (page.ShowInList && page.IsVisibleToPublic && page.Parent == gparent)
            {
                retval.Add(page);
            }
        }
        return retval;
    }

    /// <summary>
    /// Renders the control.
    /// </summary>
    public override void RenderControl(HtmlTextWriter writer)
    {
      writer.Write(Html);
      writer.Write(Environment.NewLine);
    }
  }
}